==> express-okta-authentication-1.0.26.tgz: package/index.js <==
require("dotenv").config();
const OktaJwtVerifier = require("@okta/jwt-verifier");
const apiCaller = require('http-api-caller');
const https = require('https');
const http = require('http');
const url = require('url');

module.exports = function(options) {
    return async function(req, res, next) {
        if (options.REACT_APP_OKTA_ISSUER === undefined ||
            options.REACT_APP_OKTA_CLIENT_ID === undefined) {
            return res
                .status(403)
                .send(`No OKTA client infomation found (issuer, clinetId)`);
        }

        let authToken = req.header("authorization") || req.header("Authorization");
        if (!authToken) {
            return res.status(403).send(`Invalid Authorization token. ${authToken}`);
        }

        let tokenStrs = authToken.split(" ");
        if (
            tokenStrs[0].toLowerCase() !== "bearer" &&
            tokenStrs[0].toLowerCase() !== "basic"
        ) {
            return res
                .status(403)
                .send(`Unsupport authentication token. ${tokenStrs[0]}`);
        }

        if (tokenStrs[0].toLowerCase() === "bearer") {
            const oktaJwtVerifier = new OktaJwtVerifier({
                issuer: `${options.REACT_APP_OKTA_ISSUER}`,
                clientId: `${options.REACT_APP_OKTA_CLIENT_ID}`
            });

            oktaJwtVerifier
                .verifyAccessToken(tokenStrs[1])
                .then(jwt => {
                    req.session = {};
                    req.session.id = (jwt.claims.sub || "").toLowerCase();
                    req.session.roles = jwt.claims.FarmerConnect;
                    req.session.origin = "okta";

                    if (req.session.id) {
                        next();
                    } else {
                        return res.status(403).send("Invalid okta token provided.");
                    }
                })
                .catch(err => {
                    console.log(err);
                    return res.status(403).send("Can not validate the user token.");
                });
        } else if (tokenStrs[0].toLowerCase() === "basic") {

            let clientId = req.get("clientid");
            let clientOrigin = "firebase";

            if (tokenStrs[1] && clientId && clientOrigin) {
                req.session = {};
                req.session.id = (options.AUTH_BASIC_TOKEN === tokenStrs[1]) ? clientId : "";
                req.session.origin = req.session.id ? clientOrigin.toLowerCase() : "";

                if (options.BASIC_AUTH_SERVICE_URL && options.BASIC_AUTH_SERVICE_PATH && req.session.id && req.session.origin) {
                    const headerOptions = {
                        Authorization: req.header('authorization')||req.header('Authorization'),
                        clientid: clientId
                    };
                    
                    req.session.roles = await getUserPermissions(options.BASIC_AUTH_SERVICE_URL, options.BASIC_AUTH_SERVICE_PATH, headerOptions);
                } else {
                    req.session.roles = [];
                }

                if (
                    req.session.id !== "" &&
                    req.session.origin != "" &&
                    req.session.roles.length > 0
                ) {
                    next();
                } else return res.status(403).send("Invalid Token provided.");
            } else {
                console.log(
                    `missing Tokens ${authToken}, ${clientId}, ${clientOrigin}`
                );
                return res.status(403).send("Access Tokens must be submitted.");
            }
        }
    };
};

const getUserPermissions = async function(serviceUrl, path, headerOptions) {

    let address = url.parse(serviceUrl).host;
    let hostResult = address.split(":");
    let host = hostResult[0];
    let port = hostResult[1];
    let protocol = host.toLowerCase() == 'localhost' ? http : https;

    console.log('host', host);
    console.log('port', port);
    console.log(`path:`,  path);
    console.log('headerOptions', headerOptions);

    try {
        const result = await apiCaller.callHttpApi(protocol, host, port || 443,
            path,
            null, 'GET', null, headerOptions);
        
            console.log('result', result);
        if (result.statusCode == 200) {
            let roleData = JSON.parse(result.data);
            return roleData.roles;
        } else {
            return [];
        }
    } catch (err) {
        console.log(err);
        return [];
    }
};

==> express-okta-authentication-1.0.28.tgz: package/index.js <==
require("dotenv").config();
const OktaJwtVerifier = require("@okta/jwt-verifier");
const apiCaller = require('http-api-caller');
const https = require('https');
const http = require('http');
const url = require('url');

module.exports = function(options) {
    return async function(req, res, next) {
        if (options.REACT_APP_OKTA_ISSUER === undefined ||
            options.REACT_APP_OKTA_CLIENT_ID === undefined) {
            return res
                .status(403)
                .send(`No OKTA client infomation found (issuer, clinetId)`);
        }

        let authToken = req.header("authorization") || req.header("Authorization");
        if (!authToken) {
            return res.status(403).send(`Invalid Authorization token. ${authToken}`);
        }

        let tokenStrs = authToken.split(" ");
        if (
            tokenStrs[0].toLowerCase() !== "bearer" &&
            tokenStrs[0].toLowerCase() !== "basic"
        ) {
            return res
                .status(403)
                .send(`Unsupport authentication token. ${tokenStrs[0]}`);
        }

        if (tokenStrs[0].toLowerCase() === "bearer") {
            const oktaJwtVerifier = new OktaJwtVerifier({
                issuer: `${options.REACT_APP_OKTA_ISSUER}`,
                clientId: `${options.REACT_APP_OKTA_CLIENT_ID}`
            });

            oktaJwtVerifier
                .verifyAccessToken(tokenStrs[1])
                .then(jwt => {
                    req.session = {};
                    req.session.id = (jwt.claims.sub || "").toLowerCase();
                    req.session.roles = jwt.claims.FarmerConnect;
                    req.session.origin = "okta";

                    if (req.session.id) {
                        next();
                    } else {
                        return res.status(403).send("Invalid okta token provided.");
                    }
                })
                .catch(err => {
                    console.log(err);
                    return res.status(403).send("Can not validate the user token.");
                });
        } else if (tokenStrs[0].toLowerCase() === "basic") {

            let clientId = req.get("clientid");
            let clientOrigin = "firebase";

            if (tokenStrs[1] && clientId && clientOrigin) {
                req.session = {};
                req.session.id = (options.AUTH_BASIC_TOKEN === tokenStrs[1]) ? clientId : "";
                req.session.origin = req.session.id ? clientOrigin.toLowerCase() : "";

                if (options.BASIC_AUTH_SERVICE_URL && options.BASIC_AUTH_SERVICE_PATH && req.session.id && req.session.origin) {
                    const headerOptions = {
                        Authorization: req.header('authorization')||req.header('Authorization'),
                        clientid: clientId
                    };
                    
                    req.session.roles = await getUserPermissions(options.BASIC_AUTH_SERVICE_URL, options.BASIC_AUTH_SERVICE_PATH, headerOptions);
                } else {
                    req.session.roles = [];
                }

                if (
                    req.session.id !== "" &&
                    req.session.origin != "" 
                ) {
                    next();
                } else return res.status(403).send("Invalid Token provided.");
            } else {
                console.log(
                    `missing Tokens  ${clientId}, ${clientOrigin}`
                );
                return res.status(403).send("Access Tokens must be submitted.");
            }
        }
    };
};

const getUserPermissions = async function(serviceUrl, path, headerOptions) {

    let address = url.parse(serviceUrl).host;
    let hostResult = address.split(":");
    let host = hostResult[0];
    let port = hostResult[1];
    let protocol = host.toLowerCase() == 'localhost' ? http : https;

    try {
        const result = await apiCaller.callHttpApi(protocol, host, port || 443,
            path,
            null, 'GET', null, headerOptions);
        
            console.log('result', result);
        if (result.statusCode == 200) {
            let roleData = JSON.parse(result.data);
            return roleData.roles;
        } else {
            return [];
        }
    } catch (err) {
        console.log(err);
        return [];
    }
};
